//-------------------------------------------------------------------------------
// Functions for processing surveys.
//-------------------------------------------------------------------------------

function ValidateSurvey()
{
   // Fail if text boxes are left blank.
   if( strlen( trim( txtAdjective1.getText() ) ) == 0 ) return false;
   if( strlen( trim( txtAdjective2.getText() ) ) == 0 ) return false;
   if( strlen( trim( txtAdjective3.getText() ) ) == 0 ) return false;
   if( strlen( trim( txtWhereFrom.getText() ) ) == 0 ) return false;
   if( strlen( trim( txtOccupation.getText() ) ) == 0 ) return false;
   if( strlen( trim( txtBreakfast.getText() ) ) == 0 ) return false;

   return true;
}

function GetCleanText(%txtCtrl)
{
   %text = %txtCtrl.getText();
   %text = strreplace( %text, ",", " " );
   %text = strreplace( %text, "  ", " " );
   return strlwr( trim( %text ) );
}

function GetCleanNumber(%sldCtrl)
{
   %val = "" @ %sldCtrl.getValue();
   %decimal = strstr( %val, "." );
   if( %decimal > -1 ) %val = getSubStr( %val, 0, %decimal );
   return %val;
}

function ProcessSurvey()
{
   // Validate that survey has been filled in completely.
   if( !ValidateSurvey() )
   {
      Canvas.pushDialog( MessageBoxSurveyIncompleteDlg );  
      return;
   }
 
   // Send problems to the server.
   if( strlen( txtProblems.getText() > 0 ) )
   {
      CommandToServer('LogInfo', "Problems", txtProblems.getText() );
   }

   // Send adjectives to the server.
   CommandToServer( 'LogAdjectives', txtDescribeRole.other, GetCleanText(txtAdjective1), GetCleanText(txtAdjective2), GetCleanText(txtAdjective3) );

   // Report abuse.
   if( chkAbuse.getValue() != 0 )
   {
      CommandToServer('LogInfo', "Abuse", txtDescribeRole.other );
   }

   // Build string of tagged values to send to other client via the server.
   %report = "Gender=" @ getsubstr(lstGender.getText(), 0, 1);
   %report = %report @ "," @ "Age=" @ GetCleanNumber( sldAge );
   %report = %report @ "," @ "From=" @ GetCleanText( txtWhereFrom );
   %report = %report @ "," @ "Occupation=" @ GetCleanText( txtOccupation );
   %report = %report @ "," @ "Breakfast=" @ GetCleanText( txtBreakfast );
   %report = %report @ "," @ "Intelligent=" @ GetCleanNumber( sldIntelligent );
   %report = %report @ "," @ "Funny=" @ GetCleanNumber( sldFunny );
   %report = %report @ "," @ "Considerate=" @ GetCleanNumber( sldConsiderate );
   %report = %report @ "," @ "Honest=" @ GetCleanNumber( sldHonest );
   %report = %report @ "," @ "Well Spoken=" @ GetCleanNumber( sldWellSpoken );
   %report = %report @ "," @ "Patient=" @ GetCleanNumber( sldPatient );

   // Send the profile to the server.
   CommandToServer( 'SaveProfile', txtDescribeRole.other, %report );

   // Show the personality profile results.
   dlgSurvey.setVisible(false);
   dlgProfile.setVisible(true);

   if( lstProfile.otherQuit )
   {
      btnProfileOk.setVisible(true);
      return;
   }

   if( !lstProfile.receivedReport )
   {
      btnProfileOk.setVisible(false);

      lstProfile.clear();
      lstProfile.addRow( 0, " " );
      lstProfile.addRow( 1, "YOUR PERSONALITY PROFILE WILL APPEAR HERE SHORTLY." );
      lstProfile.addRow( 2, " " );
      lstProfile.addRow( 3, " WAITING FOR OTHER PLAYER TO SUBMIT SURVEY" );
      schedule( 400, 0, AddDot );
   }
}

function AddDot()
{
   if( lstProfile.receivedReport ) return;
   if( lstProfile.otherQuit ) return;

   %text = lstProfile.getRowTextById( 3 );
   if( strlen(%text) > 100 ) %text = " WAITING FOR OTHER PLAYER TO SUBMIT SURVEY";
   else %text = %text SPC ".";
   
   lstProfile.setRowById( 3, %text );
   schedule( 400, 0, AddDot );
}

function clientCmdDisplayProfile( %report, %email )
{
   %formatted = ProcessReport( %report, %email );

   lstProfile.receivedReport = true;
   lstProfile.clear();

   %row = 0;
   %loop = true;
   while( %loop )
   {
      %formatted = NextToken( %formatted, tok, "\n" );
      lstProfile.addRow( %row, %tok );
      %row = %row + 1;
      if(%formatted $=""){%loop=false;}
   }

   btnProfileOk.setVisible(true);
}




















